///////////////////////////////////////////////////////////////////////////////
//                                                         
// Matcher.cc
// ----------
// Regular Expression handling
//                                               
// Design and Implementation by Bjoern Lemke               
//                                                         
// (C)opyright 2000-2013 Bjoern Lemke                        
//
// IMPLEMENTATION MODULE
//
// Class: Matcher
// 
// Description:
//
// Status: IMPLEMENTED
///////////////////////////////////////////////////////////////////////////////


// INCLUDES

#include <stdlib.h>

#include "Exception.h"
#include "Chain.h"
#include "Matcher.h"

Matcher::Matcher(const Chain& expr)
{    
    _expr = expr;
    _isPrepared = false;
    _pRE = 0;
}

Matcher::~Matcher()
{
    if ( _pRE )
	free(_pRE);
}

void Matcher::prepare()
{

    if ( (char*)_expr != 0 )
    {
	_pRE = (regex_t*)malloc(sizeof(regex_t));
	if ( ::regcomp(_pRE, (char*)_expr, 0) != 0 )
	{
	    throw Exception(EXLOC, "regcomp error"); 
	}
    }
    else
    {
	Chain msg = Chain("Invalid regular expression <") + _expr + Chain(">"); 
	throw Exception(EXLOC, msg); 
    }
    _isPrepared = true;    
}

bool Matcher::match(const Chain& str)
{
    if ( _isPrepared )
    {
	if ( (char*)str != 0  )
	    if (::regexec(_pRE, (char*)str, 0, NULL, 0) == 0)
		return true;
	return false;
    }
    throw Exception(EXLOC, "Matcher not prepared"); 
}




